#!/usr/local/biotools/perl/5.14.2/bin/perl

=head1 NAME
   split_bam.pl

=head1 SYNOPSIS

    USAGE: split_bam.pl -r=run_info.txt -o=output_dir -s=sample_name [-c=chridx]

=head1 OPTIONS


B<--run_info,-r>
	Required. Complete path to run info file

B<--output_dir, -o>
	Required.  Root output dir.  eg: /data2/bsi/secondary/PI/mrnaseq/RUN_ID

B<--input_dir, -i>
	Required. Location of input file

B<--sample, -s>
	Required. Sample name for which cufflinks should be executed.

B<--chr, -c>
	Option. Split input bam for a given chr.


B<--help,-h>


=head1 DESCRIPTION
	Run split bam module for a given sample

=head1 INPUT
	Run info file and sample name

=head1 OUTPUT

=head1 VERSION
	1.0

=head1  CONTACT
  bjaysheel@gmail.com


==head1 EXAMPLE
	./split_bam.pl -r=run_info.pl -o=output_dir -s=sample_name [-c=chridx]

=cut

use lib "/data2/bsi/reference/perl_workflow_ref/lib";
use lib "/data2/bsi/reference/perl_workflow_ref/lib/perl5/x86_64-linux/auto";
use strict;
use warnings;
use Data::Dumper;
use Cwd;
use Pod::Usage;
use Getopt::Long qw(:config no_ignore_case no_auto_abbrev pass_through);
use POSIX;
use ParseConfig;
use MyUtility;
use Workflow::Logger;

my %options = ();
my $results = GetOptions (\%options,
                          'run_info|r=s',
						  'output_dir|o=s',
						  'input_dir|i=s',
						  'sample|s=s',
						  'chr|c=s',
						  'log|l=s',
			              'debug=s',
						  'help|h') || pod2usage();

#### display documentation
if( $options{'help'} ){
    pod2usage( {-exitval => 0, -verbose => 2, -output => \*STDERR} );
}

#############################################################################
#### create hash of all config info.
my $config = new ParseConfig($options{run_info});
my $util = new MyUtility;

#### make sure everything passed was peachy
&check_parameters(\%options);

#### setup log object
my $logger;

if (defined $ENV{'SGE_TASK_ID'}){
	$logger = new Workflow::Logger('LOG_FILE'=>"$config->{RunInfo}->{logs}/SplitBAM.$options{sample}.$ENV{'SGE_TASK_ID'}.log",
								  'LOG_LEVEL'=>$options{'debug'});
} else {
	$logger = new Workflow::Logger('LOG_FILE'=>"$config->{RunInfo}->{logs}/SplitBAM.$options{sample}.$options{chr}.log",
							  'LOG_LEVEL'=>$options{'debug'});
}

$logger = Workflow::Logger::get_logger();

$logger->info("Split BAM for $options{sample} started");

#### set local variables.
my $cmd = "";
my $this;

$this->{output_dir} = "$options{output_dir}/lincRNA/splitBAM/$options{sample}";
$this->{input} = "$options{input_dir}/tophat_$options{sample}/accepted_hits.bam";

create_dir_struct(\%options);

check_input($this->{input});

my @chr = split(/:/, $config->{RunInfo}->{chrindex});
my $chr_idx = "";

if (defined $ENV{'SGE_TASK_ID'}){
	$chr_idx = $ENV{'SGE_TASK_ID'} - 1;
} else {
	$chr_idx = $options{chr} - 1;
}

$this->{bam} = "$this->{output_dir}/$options{sample}_sorted.chr$chr[$chr_idx].bam";

#### extract and create per chr bam file.
$cmd = "$config->{ToolInfo}->{samtools}->{value}/samtools view -h $this->{input}";
$cmd .= " chr$chr[$chr_idx] | $config->{ToolInfo}->{samtools}->{value}/samtools view -bS -";
$cmd .= " > $this->{bam}";
execute_cmd($cmd);

#### extract and create per chr bam file.
$cmd = "$config->{ToolInfo}->{samtools}->{value}/samtools index $this->{bam}";
execute_cmd($cmd);

$logger->info("Split BAM for $options{sample} complete");
exit();

#############################################################################
sub check_parameters {
    my $options = shift;

	my @required = qw(run_info output_dir sample);

	foreach my $key (@required) {
		unless (defined $options{$key}) {
			print STDERR "ARG: $key is required\n";
			pod2usage({-exitval => 2,  -message => "error message", -verbose => 1, -output => \*STDERR});
			exit(-1);
		}
	}

	$options{'debug'} = 3 unless (defined $options{'debug'});
	$options{'chr'} = 1 unless (defined $options{'chr'});
}

#############################################################################
sub create_dir_struct {
	my $options = shift;

	my $dir = "$this->{output_dir}";
	if ( -d $dir ) {
		$logger->info("Directory $dir exist");
	} else {
		execute_cmd("mkdir -p $dir");
	}
}

#############################################################################
sub execute_cmd {
	my $cmd = shift;

	$logger->info("$cmd");
	system($cmd);

	while (( $? >> 8 ) != 0 ){
		$logger->logdie("ERROR: Following command failed to execute. Exiting execution of workflow\n$cmd");
		exit(100);
	}
}

#############################################################################
sub check_input {
	my $file = shift;

	my $job_id = 0;
	my $sgeerr = "";
	my $sgeout = "";

	if (defined $ENV{JOB_ID}){
		$job_id = $ENV{JOB_ID};
	}

	if (defined $ENV{SGE_STDERR_PATH}){
		$sgeerr = $ENV{SGE_STDERR_PATH};
	}

	if (defined $ENV{SGE_STDOUT_PATH}){
		$sgeout = $ENV{SGE_STDOUT_PATH};
	}

	if (! -s $file){
		my $error_name = "$config->{RunInfo}->{base_output_dir}/$config->{RunInfo}->{pi}/$config->{RunInfo}->{type}/$config->{RunInfo}->{output_folder}/error/SPLITBAM.$options{sample}.err";
		$util->createErrorFile($error_name, "EXPECTED FILE WHILE RUNNING SPLITBAM STEP IS MISSING\n\n$file");

		$util->reportErrorSGE($config->{RunInfo}->{email},
							  $file,
							  "SplitBAM",
							  $error_name,
							  $job_id,
							  $sgeerr,
							  $sgeout);
		exit(100);
	}
}
